package com.pms.ocp.common.aspect;

import java.util.HashMap;
import java.util.Map;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import com.alibaba.fastjson.JSONObject;

import lombok.extern.slf4j.Slf4j;

/**
 * 统一接口日志记录
 *
 * @author wuwanli
 * @version 1.0
 * @date 2021/8/6
 */
@Order(1)
@Slf4j
@Aspect
@Component
public class InterfaceLogAspect {

    @Before("execution(* com.pms.ocp.controller..*.*(..))")
    public void before(JoinPoint joinPoint) {
        // 打印接口请求参数日志
        this.logInterfaceRequest(joinPoint);
    }

    @Around("execution(* com.pms.ocp.controller..*.*(..))")
    public Object around(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        long startTime = System.currentTimeMillis();
        Object result = proceedingJoinPoint.proceed();
        log.info("interface responseBody : " + (result != null ? JSONObject.toJSONString(result) : null));
        log.info("interface time-consuming :{}", System.currentTimeMillis() - startTime);
        return result;
    }

    /**
     * 打印接口请求参数日志
     *
     * @param joinPoint 连接点
     */
    private void logInterfaceRequest(JoinPoint joinPoint) {
        try {
            ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder
                    .getRequestAttributes();
            MethodSignature methodSignature = (MethodSignature) joinPoint.getSignature();
            String[] parameterNames = methodSignature.getParameterNames();
            Map<String, Object> paramMap = new HashMap<>(10);
            Object[] args = joinPoint.getArgs();
            if (parameterNames.length <= args.length) {
                for (int i = 0; i < parameterNames.length; i++) {
                    paramMap.put(parameterNames[i], args[i]);
                }
            }
            log.info("interface          url : "
                    + (attributes != null ? attributes.getRequest().getRequestURL() : null));
            log.info("interface    className : " + joinPoint.getTarget().getClass().getName());
            log.info("interface   methodName : " + joinPoint.getSignature().getName());
            log.info("interface  requestBody : " + JSONObject.toJSONString(paramMap));
        } catch (Exception exception) {
            log.error("InterfaceLogAspect.before Exception", exception);
        }
    }
}
